@@ -151,8 +151,8 @@ impl MpegTsPcr {
         let pcr = gst::ClockTime::from(self);
 
         let absdiff = pts.absdiff(pcr);
-        // Fast paths, no wraparounds and close to the PCR as it should (< 1s is required by T-STD)
-        let threshold = gst::ClockTime::from_mseconds(1500);
+        // Fast paths, no wraparounds and close (enough) to the PCR as it should (< 1s is required by T-STD)
+        let threshold = gst::ClockTime::from_mseconds(5000);
         if absdiff <= threshold {
             return Some(pts);
         }
@@ -235,10 +235,12 @@ struct Stream {
 }
 
 #[derive(Default)]
-struct State {
-    // Controlled source element
+struct Settings {
     source: Option<gst::Element>,
+}
 
+#[derive(Default)]
+struct State {
     // Last observed PCR (for handling wraparound)
     last_seen_pcr: Option<MpegTsPcr>,
 
@@ -643,7 +645,6 @@ impl State {
                         }
                         Ok(None) => break,
                         Err(err) => {
-                            dbg!(&header);
                             gst::warning!(
                                 CAT,
                                 imp = imp,
@@ -696,6 +697,7 @@ pub struct MpegTsLiveSource {
     external_clock: gst::SystemClock,
 
     state: Mutex<State>,
+    settings: Mutex<Settings>,
 }
 
 impl MpegTsLiveSource {
@@ -731,7 +733,6 @@ impl MpegTsLiveSource {
             let new_pts = self
                 .external_clock
                 .adjust_unlocked(pts + base_time)
-                .expect("Couldn't adjust {pts}")
                 .saturating_sub(base_time);
             gst::debug!(
                 CAT,
@@ -743,6 +744,8 @@ impl MpegTsLiveSource {
             buffer.set_dts(new_pts);
         };
 
+        drop(state);
+
         gst::ProxyPad::chain_default(pad, Some(&*self.obj()), buffer)
     }
 
@@ -783,7 +786,6 @@ impl MpegTsLiveSource {
                 let new_pts = self
                     .external_clock
                     .adjust_unlocked(pts + base_time)
-                    .expect("Couldn't adjust {pts}")
                     .saturating_sub(base_time);
                 gst::debug!(
                     CAT,
@@ -797,6 +799,8 @@ impl MpegTsLiveSource {
             ControlFlow::Continue(Some(buffer))
         });
 
+        drop(state);
+
         gst::ProxyPad::chain_list_default(pad, Some(&*self.obj()), bufferlist)
     }
 }
@@ -849,6 +853,7 @@ impl ObjectSubclass for MpegTsLiveSource {
             internal_clock,
             external_clock,
             state: Mutex::new(State::default()),
+            settings: Mutex::new(Settings::default()),
         }
     }
 }
@@ -880,8 +885,8 @@ impl ObjectImpl for MpegTsLiveSource {
     fn set_property(&self, _id: usize, value: &glib::Value, pspec: &glib::ParamSpec) {
         match pspec.name() {
             "source" => {
-                let mut state = self.state.lock().unwrap();
-                if let Some(existing_source) = state.source.take() {
+                let mut settings = self.settings.lock().unwrap();
+                if let Some(existing_source) = settings.source.take() {
                     let _ = self.obj().remove(&existing_source);
                     let _ = self.srcpad.set_target(None::<&gst::Pad>);
                 }
@@ -906,9 +911,9 @@ impl ObjectImpl for MpegTsLiveSource {
                         gst::warning!(CAT, imp = self, "Failed to set ghost pad target");
                         return;
                     }
-                    state.source = Some(source);
+                    settings.source = Some(source);
                 } else {
-                    state.source = None;
+                    settings.source = None;
                 }
             }
             "window-size" => {
@@ -920,7 +925,7 @@ impl ObjectImpl for MpegTsLiveSource {
 
     fn property(&self, _id: usize, pspec: &glib::ParamSpec) -> glib::Value {
         match pspec.name() {
-            "source" => self.state.lock().unwrap().source.to_value(),
+            "source" => self.settings.lock().unwrap().source.to_value(),
             "window-size" => self.external_clock.window_size().to_value(),
             _ => unimplemented!(),
         }
@@ -953,7 +958,7 @@ impl ElementImpl for MpegTsLiveSource {
     ) -> Result<gst::StateChangeSuccess, gst::StateChangeError> {
         if transition == gst::StateChange::ReadyToPaused
             && self
-                .state
+                .settings
                 .lock()
                 .expect("Couldn't get state")
                 .source
