@@ -91,7 +91,7 @@ impl FindInstance {
         unsafe { NDIlib_find_wait_for_sources(self.0.as_ptr(), timeout_in_ms) }
     }
 
-    pub fn get_current_sources(&mut self) -> Vec<Source> {
+    pub fn get_current_sources(&mut self) -> Vec<Source<'_>> {
         unsafe {
             let mut no_sources = mem::MaybeUninit::uninit();
             let sources_ptr =
@@ -402,7 +402,7 @@ pub struct SendInstance(ptr::NonNull<::std::os::raw::c_void>);
 unsafe impl Send for SendInstance {}
 
 impl SendInstance {
-    pub fn builder(ndi_name: &str) -> SendBuilder {
+    pub fn builder(ndi_name: &str) -> SendBuilder<'_> {
         SendBuilder {
             ndi_name,
             clock_video: false,
@@ -463,6 +463,7 @@ impl Tally {
 }
 
 #[derive(Debug)]
+#[allow(clippy::large_enum_variant)]
 pub enum Frame {
     Video(VideoFrame),
     Audio(AudioFrame),
@@ -473,6 +474,7 @@ pub enum Frame {
 pub struct VideoFrame(VideoFrameInner);
 
 #[derive(Debug)]
+#[allow(clippy::large_enum_variant)]
 enum VideoFrameInner {
     //Owned(NDIlib_video_frame_v2_t, Option<ffi::CString>, Option<Vec<u8>>),
     BorrowedRecv(NDIlib_video_frame_v2_t, Arc<RecvInstancePtr>),
@@ -629,7 +631,7 @@ impl VideoFrame {
     }
 
     #[cfg(feature = "advanced-sdk")]
-    pub fn compressed_packet(&self) -> Option<CompressedPacket> {
+    pub fn compressed_packet(&self) -> Option<CompressedPacket<'_>> {
         use byteorder::{LittleEndian, ReadBytesExt};
         use std::io::Cursor;
         use std::slice;
@@ -777,7 +779,9 @@ impl VideoFrame {
 
                 if (frame.plane_data(2).unwrap().as_ptr() as usize)
                     .checked_sub(frame.plane_data(1).unwrap().as_ptr() as usize)
-                    != Some((frame.height() as usize + 1) / 2 * frame.plane_stride()[1] as usize)
+                    != Some(
+                        (frame.height() as usize).div_ceil(2) * frame.plane_stride()[1] as usize,
+                    )
                 {
                     return Err(TryFromVideoFrameError);
                 }
@@ -814,7 +818,9 @@ impl VideoFrame {
 
                 if (frame.plane_data(2).unwrap().as_ptr() as usize)
                     .checked_sub(frame.plane_data(1).unwrap().as_ptr() as usize)
-                    != Some((frame.height() as usize + 1) / 2 * frame.plane_stride()[1] as usize)
+                    != Some(
+                        (frame.height() as usize).div_ceil(2) * frame.plane_stride()[1] as usize,
+                    )
                 {
                     return Err(TryFromVideoFrameError);
                 }
@@ -981,7 +987,7 @@ impl AudioFrame {
     }
 
     #[cfg(feature = "advanced-sdk")]
-    pub fn compressed_packet(&self) -> Option<CompressedPacket> {
+    pub fn compressed_packet(&self) -> Option<CompressedPacket<'_>> {
         use byteorder::{LittleEndian, ReadBytesExt};
         use std::io::Cursor;
         use std::slice;
